//
//  FilmsTableViewController.swift
//  Buscar Peliculas
//
//  Created by Santiago Pavón Gómez on 14/12/15.
//  Copyright © 2015 UPM. All rights reserved.
//

import UIKit

class FilmsTableViewController: UITableViewController {
    
    // Parametro de entrada: el nombre de la pelicula
    var filmName: String!

    // Parametro de entrada: URL Session para descargar datos
    var session: NSURLSession!

    // Array con las peliculas encontradas tras la busqueda.
    private var films = [AnyObject]()
    
    // Almacen de images de peliculas.
    // La clave es un String con el URL, y el valor es un objeto UIImage.
    private var images = [String:UIImage]()
    
    
    // Offset por el que voy bajando paginas
    private var offset = 0
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        if filmName == nil {
            filmName = "war"
        }
        
        self.title = filmName
    }

    override func viewDidAppear(animated: Bool) {
        super.viewDidAppear(animated)
        
        getFilmsWithOffset()
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        images = [:]
    }

    // MARK: - Descargar peliculas
    
    private func getFilmsWithOffset() {
        
        // No descargo mas peliculas si el ViewController no esta visible
        if view.window == nil {
            return
        }
        
        // Construye la query que hay que añadir a la URL.
        // Con esta query solo se bajan 10 pellicuals.
        let query = "title=\(filmName)&limit=10&offset=\(offset)&lang=es-es&token=\(Constants.TOKEN)"
        
        // La proxima vez bajare las siguientes 10 peliculas
        offset += 10
        
        // Escapando caracteres conflictivos
        if let escapedQuery = query.stringByAddingPercentEncodingWithAllowedCharacters(.URLQueryAllowedCharacterSet()) {
            
            let pageURL = "\(Constants.API_BASE_URL)?\(escapedQuery)"
            
            print("URL = \(pageURL)")
            
            // Descargamos un JSON con los datos.
            if let url = NSURL(string: pageURL) {
                
                self.title = "Downloading ..."
                NetActivity.shared.incrNetActivity()
                
                let task = session.dataTaskWithURL(url) {(data: NSData?, res: NSURLResponse?, error: NSError?) in
                    dispatch_async(dispatch_get_main_queue(), {
                        if error != nil {
                            self.title = "Error"
                            print("Error 1: \(error?.localizedDescription)")
                        } else {
                            let code = (res as! NSHTTPURLResponse).statusCode
                            if code == 200 {
                                // Extraer el array de datos del JSON
                                do {
                                    
                                   // let str = NSString(data: data!, encoding: NSUTF8StringEncoding)
                                   // print("Datos descargados: \(str!)")
                                    
                                    if let dic = try NSJSONSerialization.JSONObjectWithData(data!, options: []) as? [String:AnyObject] {
                                        if let dataDic = dic["data"] as? [String:AnyObject] {
                                            if let moviesArr = dataDic["movies"] as? [AnyObject] {
                                                
                                                if moviesArr.count > 0 {
                                                    self.films += moviesArr
                                                    self.tableView.reloadData()
                                                    // Sigo bajando peliculas.
                                                    self.getFilmsWithOffset()
                                                }
                                                self.title = self.filmName
                                            } else {
                                                self.title = "Error"
                                                print("Error 7: El JSON es rarito.")
                                            }
                                        } else if let errDic = dic["error"] as? [String:AnyObject] {
                                            self.title = "Error"
                                            print("Error 5: \(errDic["message"] ?? "Desconocido")")
                                        } else {
                                            self.title = "Error"
                                            print("Error 6: El JSON es rarito.")
                                        }
                                    }
                                } catch let err as NSError {
                                    self.title = "Error"
                                    print("Error 2: \(err.localizedDescription)")
                                } catch {
                                    self.title = "Error"
                                    print("Error 3")
                                }
                            } else {
                                self.title = "Error"
                                print("Error 4: \(NSHTTPURLResponse.localizedStringForStatusCode(code))")
                            }
                        }
                        NetActivity.shared.decrNetActivity()
                    })
                }
                task.resume()
            }
        } else {
            self.title = "Error"
            print("Error 8")
        }
    }
    
    // MARK: - Table view data source

    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 1
    }

    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return films.count
    }

    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("Film Cell", forIndexPath: indexPath)

        cell.textLabel?.text = ""
        cell.detailTextLabel?.text = ""
        cell.imageView?.image = nil
        
        if let dic = films[indexPath.row] as? [String:AnyObject] {
            if let title = dic["title"] as? String {
                cell.textLabel?.text = title
            }
            if let year = dic["year"] as? String {
                cell.detailTextLabel?.text = year
            }
            if let posterUrl = dic["urlPoster"] as? String {
                if let image = images[posterUrl] {
                    cell.imageView?.image = image
                } else {
                    if let url = NSURL(string: posterUrl) {
                        let task = session.dataTaskWithURL(url) {(data: NSData?, res: NSURLResponse?, error: NSError?) in
                            if error == nil && (res as! NSHTTPURLResponse).statusCode == 200 {
                                if let image = UIImage(data: data!) {
                                    dispatch_async(dispatch_get_main_queue(), {
                                        // Guardar la imagen
                                        self.images[posterUrl] = image
                                        
                                        // Actualizar la celda
                                        self.tableView.reloadRowsAtIndexPaths([indexPath], withRowAnimation: .Fade)
                                    })
                                }
                            }
                            dispatch_async(dispatch_get_main_queue(), {
                                NetActivity.shared.decrNetActivity()
                            })
                        }
                        NetActivity.shared.incrNetActivity()
                        task.resume()
                    }
                }
            }
        }
        
        return cell
    }

    // MARK: - Navegar

    
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        
        switch segue.identifier! {
        case "Show Poster":
            if let pvc = segue.destinationViewController as? PosterViewController {
                if let ip = tableView.indexPathForSelectedRow {
                    if let dic = films[ip.row] as? [String:AnyObject] {
                        if let title = dic["title"] as? String {
                            pvc.filmName = title
                        }
                        if let posterUrl = dic["urlPoster"] as? String {
                            if let image = images[posterUrl] {
                                pvc.posterImage = image
                            }
                        }
                    }
                }
            }
        case "Show Actors":
            if let atvc = segue.destinationViewController as? ActorsTableViewController {
                if let cell = sender as? UITableViewCell,
                    let ip = tableView.indexPathForCell(cell) {
                        if let dic = films[ip.row] as? [String:AnyObject] {
                            atvc.filmName = dic["title"] as? String
                            atvc.idIMDB = dic["idIMDB"] as? String
                            atvc.session = session
                        }
                }
            }
        default:
            break
        }
    }
    
}
